﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EntityDataModel;
using System.Data.Objects;

namespace CompiledQueriesCSharp
{

  public class DataRetrievalEF
  {

    //declare static funcs to contain pre-compiled queries
    //1) takes an int parameter (an ID) and returns a customer
    static Func<SalesEntities, int, Customer> _custByID;
    //2) no parameters, returns a customer graph
    static Func<SalesEntities, Customer> _randomCustomer;
    //3) no parameters, returns an IQueryable of CustomerSales type (defined below)
    static Func<SalesEntities, IQueryable<CustomerSales>> _custSales;

    static SalesEntities _context;

    public DataRetrievalEF()
    {
      _context = new SalesEntities();
    }

    public Customer GetCustomer(int ID)
    {
      //test for an existing instance of the compiled query
      if (_custByID == null)
      {
        _custByID = CompiledQuery.Compile<SalesEntities, int, Customer>
         ((ctx, id) => ctx.Customers.Where(c => c.CustomerID == id).Single());
      }
      //invoke
      return _custByID.Invoke(_context, ID);
    }

    public Customer GetRandomCustomerWithOrders()
    {

      if (_randomCustomer == null)
      {
        _randomCustomer = CompiledQuery.Compile<SalesEntities, Customer>
         (ctx => ctx.Customers.Include("Orders.Details")
                          .Where(c => c.Orders.Any())
                          .FirstOrDefault());
      }
      return _randomCustomer.Invoke(_context);
    }

    public IQueryable<CustomerSales> GetCustomerSales()
    {
      //test for an existing instance of the compiled query
      if (_custSales == null)
      {
        _custSales = CompiledQuery.Compile<SalesEntities, IQueryable<CustomerSales>>
         (ctx => from c in ctx.Customers
                 select new CustomerSales { CustomerID = c.CustomerID, TotalSales = c.Orders.Sum(o => o.SubTotal) }
                 );
      }
      return _custSales.Invoke(_context);
    }


  }
  
  //custom class for use in projected query
  public class CustomerSales
  {
    public int CustomerID { get; set; }
    public string Name { get; set; }
    public Nullable<decimal> TotalSales { get; set; }
  }
 
}
