﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using LINQtoSQLClasses;
using System.Data.Linq;

namespace CompiledQueriesCSharp
{

  public class DataRetrievalL2S
  {
   
    //takes an int parameter (an ID) and returns a cutomer
    static Func<DataClasses1DataContext, int, Customer> _custByID;
    //no parameter, returns a customer graph
    static Func<DataClasses1DataContext, Customer> _CustWithOrders;
    //no parameter, returns an IQueryable of Customers
    static Func<DataClasses1DataContext, IQueryable<Customer>> _multipleCustWithOrders;
   //no param, CustomerSalesStruct used for projections
    static Func<DataClasses1DataContext, IQueryable<CustomerSalesStruct>> _custSales;
    
    //data load options to ensure load options (e.g. lazy loading) are retained along
    //with compiled query
    static DataLoadOptions Load_Customer_Orders_Details;


    static DataClasses1DataContext _context;

    public DataRetrievalL2S()
    {
      _context = new DataClasses1DataContext();
    }

    public Customer GetCustomer(int ID)
    {
      //test for an existing instance of the compiled query
      if (_custByID == null)
      {
        _custByID = CompiledQuery.Compile<DataClasses1DataContext, int, Customer>
         ((ctx, id) => ctx.Customers.Where(c => c.CustomerID == id).Single());
      }
      return _custByID.Invoke(_context, ID);
    }

    public Customer GetRandomCustomerWithOrders()
    {
      //test for an existing instance of the compiled query
      if (Load_Customer_Orders_Details == null)
      {
        Load_Customer_Orders_Details = new DataLoadOptions();
        Load_Customer_Orders_Details.LoadWith<Customer>(c => c.Orders);
        //Load_Customer_Orders_Details.LoadWith<Order>(o => o.Details);
        //Load_Customer_Orders_Details.LoadWith<Detail>(d => d.Product);
      }
      if (_context.LoadOptions == null)
      {
        _context.LoadOptions = Load_Customer_Orders_Details;
      }
      if (_CustWithOrders == null)
      {
        _CustWithOrders = CompiledQuery.Compile<DataClasses1DataContext, Customer>
               (ctx => ctx.Customers.Where(c => c.Orders.Any()).FirstOrDefault());
        //  _context.LoadOptions = origLoadOptions;
      }

      Customer result = _CustWithOrders.Invoke(_context);
      return result;

    }

    public Customer SimpleQuery()
    {
      Load_Customer_Orders_Details = new DataLoadOptions();
      Load_Customer_Orders_Details.LoadWith<Customer>(c => c.Orders);
      Load_Customer_Orders_Details.LoadWith<Order>(o => o.Details);
      Load_Customer_Orders_Details.LoadWith<Detail>(d => d.Product);
      _context.LoadOptions = Load_Customer_Orders_Details;
      
      return _context.Customers.Single(c => c.CustomerID == 6);
    
    }
    public List<Customer> SimpleQueryManyItems()
    {
      using (DataClasses1DataContext context = new DataClasses1DataContext())
      {
        Load_Customer_Orders_Details = new DataLoadOptions();
        Load_Customer_Orders_Details.LoadWith<Customer>(c => c.Orders);
        //Load_Customer_Orders_Details.LoadWith<Order>(o => o.Details);
        //Load_Customer_Orders_Details.LoadWith<Detail>(d => d.Product);
        context.LoadOptions = Load_Customer_Orders_Details;

        return context.Customers.Where(c => c.Orders.Any()).ToList();
      }

    }
    public Customer GetMultipleCustomerWithOrders()
    {
      //test for an existing instance of the compiled query
      if (_multipleCustWithOrders == null)
      {
        DataLoadOptions origLoadOptions = _context.LoadOptions;

        Load_Customer_Orders_Details.LoadWith<Customer>(c => c.Orders);
        Load_Customer_Orders_Details.LoadWith<Order>(o => o.Details);
        Load_Customer_Orders_Details.LoadWith<Detail>(d => d.Product);
        _context.LoadOptions = Load_Customer_Orders_Details;
        _multipleCustWithOrders = CompiledQuery.Compile<DataClasses1DataContext, IQueryable<Customer>>
               (ctx => ctx.Customers.Where(c => c.Orders.Any()).Take(1));
        //  _context.LoadOptions = origLoadOptions;
      }

      var result = _multipleCustWithOrders.Invoke(_context);
      return result.Single();

    }
    public IQueryable<CustomerSalesStruct> GetCustomerSales()
    {
      //test for an existing instance of the compiled query
      if (_custSales == null)
      {
        _custSales = CompiledQuery.Compile<DataClasses1DataContext, IQueryable<CustomerSalesStruct>>
         (ctx => from c in ctx.Customers
                 select new CustomerSalesStruct { CustomerID = c.CustomerID, TotalSales = c.Orders.Sum(o => o.SubTotal) }
                 );
      }
      return _custSales.Invoke(_context);
    }



  }
  public struct CustomerSalesStruct
  {
    public int CustomerID;
    public string Name;
    public Nullable<decimal> TotalSales;
  }


}
